odoo.define('sun_pos_invoice_reprint.pos', function (require) {

var gui = require('point_of_sale.gui');
var screens = require('point_of_sale.screens');
var models = require('point_of_sale.models')
var PopupWidget = require('point_of_sale.popups');
var core = require('web.core');
var DB = require('point_of_sale.DB');

var qweb = core.qweb;
var _t = core._t;

	models.load_models([{
            model: 'account.invoice',
            fields: ['partner_id', 'origin', 'number', 'payment_term_id', 'date_invoice', 'state', 
            'residual', 'amount_tax', 'amount_total', 'type','user_id','date_due'],
            domain: function(self){ 
                console.log("self",self)
                console.log("self.user",self.user);
                return [['user_id','=',self.user.id ]]; 
            },
            context: {'pos': true},
            loaded: function (self, invoices) {
                self.db.save_invoices(invoices);
            }
        },{
            model: 'account.invoice.line',
            fields: [
                'invoice_id',
                'uom_id',
                'product_id',
                'price_unit',
                'price_subtotal',
                'quantity',
                'discount',
            ],
            domain: [],
            context: {'pos': true},
            loaded: function (self, invoice_lines) {
                self.db.save_invoice_lines(invoice_lines);
            }
        },
    ]);

	var InvoicesButton = screens.ActionButtonWidget.extend({
        template: 'InvoicesButton',
        button_click: function(){
            var order    = this.pos.get_order();
            this.gui.show_screen('invoices');
        },
    });

    screens.define_action_button({
        'name': 'invoices',
        'widget': InvoicesButton,
    });

	var invoices_screen = screens.ScreenWidget.extend({ // invoices screen
        template: 'invoices_screen',
        start: function () {
            var self = this;
            this._super();
            this.pos.bind('update:invoice', function () {
                self.render_screen();
            })
        },
        show: function () {
            var self = this;
            this.render_screen();
            this.invoice_selected = null;
            this._super();
            this.$('.invoice-list').delegate('.invoice-line', 'click', function (event) {
                self.invoice_select(event, $(this), parseInt($(this).data('id')));
            });
            this.$('.searchbox .search-invoice').click(function () {
                self.clear_search();
            });
            this.$('.invoices_open').click(function () {
                var invoices = _.filter(self.pos.db.invoices, function (invoice) {
                    return invoice.state == 'open';
                });
                self.$('.invoice_list_screen_widget').css('height','100%');
                if (invoices) {
                    var contents = self.$('.invoice-details-contents');
                    contents.empty();
                    return self.render_invoice_list(invoices);
                } else {
                    return self.pos.gui.show_popup('confirm', {
                        title: 'Warning',
                        body: 'Your database have not any invoices state at Open'
                    })
                }

            })
            var invoices = [];
            for (var i = 0; i < this.pos.db.invoices.length; i++) {
                var invoice = this.pos.db.invoices[i];
                var partner = this.pos.db.get_partner_by_id(invoice.partner_id[0]);
                if (!partner) {
                    continue;
                }
                var label = invoice['number'];
                if (invoice['name']) {
                    label += ', ' + invoice['name'];
                }
                if (partner['display_name']) {
                    label += ', ' + partner['display_name']
                }
                if (partner['email']) {
                    label += ', ' + partner['email']
                }
                if (partner['phone']) {
                    label += ', ' + partner['phone']
                }
                if (partner['mobile']) {
                    label += ', ' + partner['mobile']
                }
                invoices.push({
                    value: invoice['id'],
                    label: label
                })
            }
            var $search_box = $('.clientlist-screen .search-invoice >input');
            // $search_box.autocomplete({
            //     source: invoices,
            //     minLength: this.pos.minLength,
            //     select: function (event, ui) {
            //         if (ui && ui['item'] && ui['item']['value']) {
            //             var invoice = self.pos.db.invoice_by_id[ui['item']['value']];
            //             if (invoice) {
            //                 self.display_invoice_details(invoice);
            //             }
            //             self.clear_search();

            //         }
            //     }
            // });
        },
        invoice_select: function (event, $invoice, id) {
            var invoice = this.pos.db.get_invoice_by_id(id);
            this.$('.invoice-line').removeClass('highlight');
            $invoice.addClass('highlight');
            this.display_invoice_details(invoice);

        },

        display_invoice_details: function (invoice) {
            this.invoice_selected = invoice;
            var self = this;
            var contents = this.$('.invoice-details-contents');
            contents.empty();
            this.$('.invoice_list_screen_widget').css('height','150px');
            invoice.link = window.location.origin + "/web#id=" + invoice.id + "&view_type=form&model=account.invoice";
            contents.append($(qweb.render('invoice_detail', {widget: this, invoice: invoice})));
            var account_invoice_lines = this.pos.db.invoice_lines_by_invoice_id[invoice.id];
            if (account_invoice_lines) {
                var line_contents = this.$('.invoice_lines_detail');
                line_contents.empty();
                line_contents.append($(qweb.render('account_invoice_lines', {
                    widget: this,
                    account_invoice_lines: account_invoice_lines
                })));
            }
            this.$('.inv-print-invoice').click(function () { // print invoice
                self.chrome.do_action('account.account_invoices', {
                    additional_context: {
                        active_ids: [self.invoice_selected['id']]
                    }
                })
            });
            this.$('.inv-action_invoice_open').click(function () { // action inv open
                self.link = window.location.origin + '/web#id=' + self.invoice_selected['id'] + '&view_type=form&model=account.invoice';
                return self.gui.show_popup('confirm', {
                    title: 'Done',
                    body: 'Click open new tab review invoice',
                    confirm: function () {
                        window.open(self.link, '_blank');
                    }
                });
            });
        },
        render_screen: function () {
            this.pos.invoice_selected = null;
            var self = this;
            if (this.pos.db.invoices.length) {
                this.render_invoice_list(this.pos.db.invoices);
            }
            var search_timeout = null;
            if (this.pos.config.iface_vkeyboard && this.chrome.widget.keyboard) {
                this.chrome.widget.keyboard.connect(this.$('.searchbox input'));
            }
            this.$('.search-invoice input').on('keypress', function (event) {
                clearTimeout(search_timeout);
                var query = this.value;
                search_timeout = setTimeout(function () {
                    self.perform_search(query, event.which === 13);
                }, 70);
            });
            this.$('.searchbox .search-clear').click(function () {
                self.clear_search();
            });
            this.$('.back').click(function () {
                self.gui.show_screen('products');
            });
        },
        perform_search: function (query, associate_result) {
            if (query) {
                var invoices = this.pos.db.search_invoice(query);
                this.render_invoice_list(invoices);
            }
        },
        clear_search: function () {
            var contents = this.$('.invoice-details-contents');
            contents.empty();
            var invoices = this.pos.db.invoices;
            this.render_invoice_list(invoices);
            this.$('.searchbox input')[0].value = '';
            this.$('.searchbox input').focus();
        },
        partner_icon_url: function (id) {
            return '/web/image?model=res.partner&id=' + id + '&field=image_small';
        },
        render_invoice_list: function (invoices) {
            var contents = this.$el[0].querySelector('.invoice-list');
            contents.innerHTML = "";
            for (var i = 0, len = Math.min(invoices.length, 1000); i < len; i++) {
                var invoice = invoices[i];
                var invoice_html = qweb.render('invoice_row', {
                    widget: this,
                    invoice: invoice
                });
                invoice = document.createElement('tbody');
                invoice.innerHTML = invoice_html;
                invoice = invoice.childNodes[1];
                contents.appendChild(invoice);
            }
        }
    });
    gui.define_screen({name: 'invoices', widget: invoices_screen});

    DB.include({
    	 init: function (options) {
            this._super(options);
            // account invoices
            this.invoices = [];
            this.invoice_by_id = {};
            this.invoices = [];
            this.invoice_by_partner_id = {};
            this.invoice_search_string = "";
            this.invoice_lines_by_invoice_id = {};
        },
    	_invoice_search_string: function (invoice) {
            var str = invoice.number;
            str += '|' + invoice.name;
            if (invoice.partner_id) {
                var partner = this.partner_by_id[invoice.partner_id[0]]
                if (partner) {
                    if (partner['name']) {
                        str += '|' + partner['name'];
                    }
                    if (partner.mobile) {
                        str += '|' + partner['mobile'];
                    }
                    if (partner.phone) {
                        str += '|' + partner['phone'];
                    }
                    if (partner.email) {
                        str += '|' + partner['email'];
                    }
                }
                str += '|' + invoice['origin'];
            }
            if (invoice.date_invoice) {
                str += '|' + invoice['date_invoice'];
            }
            str = '' + invoice['id'] + ':' + str.replace(':', '') + '\n';
            return str;
        },
        search_invoice: function (query) {
            try {
                query = query.replace(/[\[\]\(\)\+\*\?\.\-\!\&\^\$\|\~\_\{\}\:\,\\\/]/g, '.');
                query = query.replace(' ', '.+');
                var re = RegExp("([0-9]+):.*?" + query, "gi");
            } catch (e) {
                return [];
            }
            var results = [];
            for (var i = 0; i < this.limit; i++) {
                var r = re.exec(this.invoice_search_string);
                if (r && r[1]) {
                    var id = r[1];
                    if (this.invoice_by_id[id] !== undefined) {
                        results.push(this.invoice_by_id[id]);
                    }
                } else {
                    break;
                }
            }
            return results;
        },
        save_invoices: function (invoices) {
            if (this.invoices.length == 0) {
                this.invoices = invoices;
            } else {
                this.invoices = this.invoices.concat(invoices);
            }
            for (var i = 0; i < invoices.length; i++) {
                var inv = invoices[i];
                this.invoice_by_id[inv.id] = inv;
                if (!this.invoice_by_partner_id[inv.partner_id[0]]) {
                    this.invoice_by_partner_id[inv.partner_id[0]] = [inv]
                } else {
                    this.invoice_by_partner_id[inv.partner_id[0]].push(inv);
                }
                this.invoice_search_string += this._invoice_search_string(inv);
            }
        },
        save_invoice_lines: function (lines) {
            for (var i = 0; i < lines.length; i++) {
                var line = lines[i];
                if (!this.invoice_lines_by_invoice_id[line.invoice_id[0]]) {
                    this.invoice_lines_by_invoice_id[line.invoice_id[0]] = [line]
                } else {
                    this.invoice_lines_by_invoice_id[line.invoice_id[0]].push(line)
                }
            }
        },
        save_data_sync_invoice_line: function (line) {
            var invoice_id = line.invoice_id[0];
            var invoice_lines = this.invoice_lines_by_invoice_id[invoice_id];
            if (!invoice_lines) {
                this.invoice_lines_by_invoice_id[invoice_id] = [line];
            } else {
                var exist = false;
                for (var i = 0; i < invoice_lines.length; i++) {
                    var old_line = invoice_lines[i];
                    if (old_line.id == line.id) {
                        old_line = line;
                        exist = true
                    }
                }
                if (exist == false) {
                    this.invoice_lines_by_invoice_id[invoice_id].push(line);
                }
            }
        },
        get_invoice_by_id: function (id) {
            return this.invoice_by_id[id];
        },
    });
});
